{%- extends "docs/base.html" %}

{% block breadcrumbs %}
<nav class="breadcrumb" aria-label="breadcrumbs">
  <ul>
    {% for parent in page.parents %}
    <li>
      <a href="{{ url_for(site_name + '.from_sitemap', path=parent.path) }}">
        {{ parent.title }}
      </a>
    </li>
    {% endfor %}
    <li class="is-active">
      <a href="{{ url_for(site_name + '.from_sitemap', path=page.path) }}" aria-current="page">
        {{ page.title }}
      </a>
    </li>
  </ul>
</nav>

{% endblock breadcrumbs %}

{% block page_content %}
  {{ content }}
{% endblock page_content %}
