{%- extends "base/base.html" %}

{% block addl_head %}
  <link rel="stylesheet" href="{{ url_for('docs.static', filename='css/docs.css')}}" />
  <link rel="stylesheet" href="{{ url_for('docs.static', filename='css/codehilite.css')}}" />
{% endblock addl_head %}


{% block content %}
<form action="/search" method="GET">
  <div class="field has-addons">
    <div class="control">
      <label for="q" class="hidden-label">Query</label>
      <input
        class="input is-small"
        type="text"
        name="q"
        placeholder="Search arXiv Help"
        {% if q %}value="{{ q }}"{% endif %}
      />
    </div>
    <button class="button is-link is-small">Search</button>
  </div>
</form>
<div class="">
  {% block breadcrumbs %}
  {% endblock breadcrumbs %}
</div>
<div class="section">
  <div class="container">
    {% block page_content %}

    {% endblock page_content %}
  </div>
</div>
{% endblock content %}
